/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
* Version: NPL 1.1/GPL 2.0/LGPL 2.1
*
* The contents of this file are subject to the Netscape Public License
* Version 1.1 (the "License"); you may not use this file except in
* compliance with the License. You may obtain a copy of the License at
* http://www.mozilla.org/NPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
* for the specific language governing rights and limitations under the
* License.
*
* The Original Code is mozilla.org code.
*
* The Initial Developer of the Original Code is 
* Netscape Communications Corporation.
* Portions created by the Initial Developer are Copyright (C) 1998
* the Initial Developer. All Rights Reserved.
*
* Contributor(s):
*
* Alternatively, the contents of this file may be used under the terms of
* either the GNU General Public License Version 2 or later (the "GPL"), or 
* the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
* in which case the provisions of the GPL or the LGPL are applicable instead
* of those above. If you wish to allow use of your version of this file only
* under the terms of either the GPL or the LGPL, and not to allow others to
* use your version of this file under the terms of the NPL, indicate your
* decision by deleting the provisions above and replace them with the notice
* and other provisions required by the GPL or the LGPL. If you do not delete
* the provisions above, a recipient may use your version of this file under
* the terms of any one of the NPL, the GPL or the LGPL.
*
* ***** END LICENSE BLOCK ***** */

////////////////////////////////////////////////////////////
//
// Implementation of plugin entry points (NPP_*)
// most are just empty stubs for this particular plugin 
//

#include <glib.h>
#include <glib/gprintf.h>
#include <cstring>
#include <stdio.h>
#include "npapi.h"
#include "npfunctions.h"
#include "AudioPlugin.h"
#include "audio_npapi.h"
#include "webbr_log.h"

char* MimeType=(char *)"audio/x-aac:unk:AUDIO Plugin;" \
                       "audio/aac:unk:AUDIO Plugin;" \
                       "audio/wav:unk:AUDIO Plugin;" \
                       "audio/vnd.wav:unk:AUDIO PLUGIN;" \
                       "audio/x-wav:unk:AUDIO PLUGIN;" \
                       "audio/x-arib-mpeg2-aac:unk:AUDIO Plugin";

#ifdef __cplusplus
extern "C" {
#endif
/*****************************************************************************
 Defines (For Debug)
******************************************************************************/
#undef DEBUG
#undef DEBUG_PRINT_FUNC_ENTRY
#undef DEBUG_PRINT_FUNC_EXIT


#if NDEBUG
  #undef  DEBUG
#else
  #undef  DEBUG
  #define DEBUG 1
#endif

#if DEBUG
#define DEBUG_PRINT_FUNC_ENTRY {\
  WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_INFO,"AudioPlugin::%s ++ Entry ++ Line:%d", __FUNCTION__, __LINE__);}
#define DEBUG_PRINT_FUNC_EXIT {\
  WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_INFO,"AudioPlugin::%s -- Exit  -- Line:%d", __FUNCTION__, __LINE__); }
#define DEBUG_PRINT_SCRIPTABLE_FUNC_ENTRY {\
  WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_INFO,"ScriptablePluginObject::%s ++ Entry ++ Line:%d", __FUNCTION__, __LINE__);}
#define DEBUG_PRINT_SCRIPTABLE_FUNC_EXIT {\
  WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_INFO,"ScriptablePluginObject::%s -- Exit  -- Line:%d", __FUNCTION__, __LINE__); }
#define CLASS_DEBUG(classname, fmt, ...) {\
  WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_DETAIL1,"%s::%s " fmt, classname, __FUNCTION__ , ## __VA_ARGS__);}
#define CLASS_WARNING(classname, fmt, ...) {\
  WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_WARNING,"%s::%s " fmt, classname, __FUNCTION__ , ## __VA_ARGS__);}
#define CLASS_ERROR(classname, fmt, ...) {\
  WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_ERROR,"%s::%s " fmt, classname, __FUNCTION__ , ## __VA_ARGS__);}
#define AUDIO_DEBUG(fmt, ...) {\
  CLASS_DEBUG("AudioPlugin", fmt , ## __VA_ARGS__);}
#define AUDIO_WARNING(fmt, ...) {\
  CLASS_WARNING("AudioPlugin", fmt , ## __VA_ARGS__);}
#define AUDIO_ERROR(fmt, ...) {\
  CLASS_ERROR("AudioPlugin", fmt , ## __VA_ARGS__);}

#else
#define DEBUG_PRINT_FUNC_ENTRY
#define DEBUG_PRINT_FUNC_EXIT
#define DEBUG_PRINT_SCRIPTABLE_FUNC_ENTRY
#define DEBUG_PRINT_SCRIPTABLE_FUNC_EXIT
#define CLASS_DEBUG(...)
#define CLASS_ERROR(...)
#define CLASS_WARNING(...)
#define AUDIO_DEBUG(...)
#define AUDIO_WARNING(...)
#define AUDIO_ERROR(...)

#endif



//For Scriptable Object
NPObject*  getInternalValue(NPP instance, NPPVariable variable);

char* NPP_Audio_GetMIMEDescription(void) {
  DEBUG_PRINT_FUNC_ENTRY
  WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_INFO, "NPP_Audio_GetMIMEDescription::MiMeType:%s", MimeType);
  DEBUG_PRINT_FUNC_EXIT
  return MimeType;
}

NPError NPP_Audio_Initialize(void) {
  NPError err = NPERR_NO_ERROR;
  DEBUG_PRINT_FUNC_ENTRY
  DEBUG_PRINT_FUNC_EXIT
  return err;
}

void NPP_Audio_Shutdown(void) {
  DEBUG_PRINT_FUNC_ENTRY
  DEBUG_PRINT_FUNC_EXIT
}

NPError NPP_Audio_New(NPMIMEType pluginType,
                      NPP instance,
                      uint16_t mode,
                      int16_t argc,
                      char* argn[],
                      char* argv[],
                      NPSavedData* saved) {
  DEBUG_PRINT_FUNC_ENTRY

  NPError err = NPERR_INVALID_PARAM;

/*
 * Audio plug-in create
 *
 */
  if (pluginType && instance && argn && argv) {
    char*	typeElement = reinterpret_cast<char*>(pluginType);
    if (typeElement) {
      if ((strncmp(typeElement, "audio/x-aac", strlen(typeElement)) == 0) ||
          (strncmp(typeElement, "audio/aac", strlen(typeElement)) == 0) ||
          (strncmp(typeElement, "audio/vnd.wav", strlen(typeElement)) == 0) ||
          (strncmp(typeElement, "audio/wav", strlen(typeElement)) == 0) ||
          (strncmp(typeElement, "audio/x-wav", strlen(typeElement)) == 0) ||
          (strncmp(typeElement, "audio/x-arib-mpeg2-aac", strlen(typeElement)) == 0)) {
        AudioPlugin* pPlugin = new AudioPlugin(instance);

        if(pPlugin) {
          char* dataElement = 0;
          for (int16_t i = 0; i < argc; i++) {
            if (strncasecmp("data", argn[i], sizeof(*argn[i])) == 0) {
              dataElement = argv[i];
            }
          }
          if ( dataElement)  {
            AUDIO_DEBUG("????set element called");
            pPlugin->setDataElement(dataElement);
            pPlugin->setTypeElement(typeElement);
          }
          err = pPlugin->init();
          instance->pdata = (void *)pPlugin;
          err = NPERR_NO_ERROR;
        } else {
          err = NPERR_GENERIC_ERROR;
        }
      } else {
        err = NPERR_INVALID_PLUGIN_ERROR;
      }
    } //type element
  } //Mime Type Check

  DEBUG_PRINT_FUNC_EXIT
  return err;
}

NPError NPP_Audio_Destroy(NPP instance, NPSavedData** save) {
  DEBUG_PRINT_FUNC_ENTRY
    NPError retval = NPERR_NO_ERROR;
  if (instance) {
    AudioPlugin* pPlugin = static_cast<AudioPlugin*>(instance->pdata);
    if (pPlugin) {
      pPlugin->destroy();
      delete pPlugin;
    }
  } else {
    retval = NPERR_INVALID_INSTANCE_ERROR;
  }
  
  DEBUG_PRINT_FUNC_EXIT
    return retval;
}

NPError NPP_Audio_SetWindow(NPP instance, NPWindow* pNPWindow) {
  DEBUG_PRINT_FUNC_ENTRY

  if (instance == NULL)	{
  DEBUG_PRINT_FUNC_EXIT
  return NPERR_INVALID_INSTANCE_ERROR;
 }

  if (pNPWindow == NULL) {
   DEBUG_PRINT_FUNC_EXIT
   return NPERR_GENERIC_ERROR;
 }

  DEBUG_PRINT_FUNC_EXIT
  return NPERR_NO_ERROR;
}

#if NP_VERSION_MAJOR==0 && NP_VERSION_MINOR<=11
#define NPP void*
#endif

NPError	NPP_Audio_GetValue(NPP instance, NPPVariable variable, void *value) {
  WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_INFO,"AudioPlugin GetValue!! %p %d %p\n", instance, variable, value);
  NPError rv = NPERR_GENERIC_ERROR;
  if (value) {
    switch (variable)
      {
      case NPPVpluginNameString:
        *((char **)value) = AUDIO_PLUGIN_NAME;
        WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_INFO,"\n Plugin Name: %s \n", *((char **)value));
        rv = NPERR_NO_ERROR;
        break;
      case NPPVpluginDescriptionString:
        *((char **)value) = AUDIO_PLUGIN_NAME;
        WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_INFO,"\n Plugin Description: %s \n",*((char **)value));
        rv = NPERR_NO_ERROR;
        break;
      case NPPVpluginScriptableNPObject:
        *(NPObject **)value =  getInternalValue(instance,variable);
        rv = NPERR_NO_ERROR;
        break;
      default:
        WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_DETAIL1,"%s default: variable=%d\n", __FUNCTION__, variable);
        rv = NPERR_GENERIC_ERROR;
      }
  }
  return rv;
}

NPObject*  getInternalValue(NPP instance, NPPVariable variable) {
  NPObject* object_value = NULL;
  if (instance) {
    AudioPlugin* pPlugin = static_cast<AudioPlugin*>(instance->pdata);
    if (pPlugin) {
      object_value = pPlugin->getScriptableInstance();
    } else  {
      AUDIO_DEBUG("base getvalue fail");
    }
  }
  DEBUG_PRINT_FUNC_ENTRY
  return object_value;
}


#if NP_VERSION_MAJOR==0 && NP_VERSION_MINOR<=11
#undef NPP
#endif

NPError NPP_Audio_NewStream(NPP instance,
                            NPMIMEType type,
                            NPStream* stream,
                            NPBool seekable,
                            uint16_t* stype) {

  DEBUG_PRINT_FUNC_ENTRY

  NPError rv = NPERR_GENERIC_ERROR;

  if ((instance == NULL) ||(type == NULL)||(stream == NULL)) {
    AUDIO_DEBUG(" Invalid Intsance (or) Mime Type (or) stream ");
    DEBUG_PRINT_FUNC_EXIT
      return NPERR_INVALID_INSTANCE_ERROR;
  }

  AudioPlugin* pPlugin = static_cast<AudioPlugin*>(instance->pdata);
  if (!pPlugin) {
    AUDIO_WARNING("Invalid Plugin Error in NPP_NewStream ");
    DEBUG_PRINT_FUNC_EXIT
      rv = NPERR_INVALID_PLUGIN_ERROR;
  } else {
    rv = pPlugin->newStream(type, stream, seekable, stype);
  }
  DEBUG_PRINT_FUNC_EXIT
    return rv;
}

int32_t NPP_Audio_WriteReady(NPP instance, NPStream *stream) {
  DEBUG_PRINT_FUNC_ENTRY

  int32_t iMaxNum = 0;

  if ((instance == NULL) || (stream == NULL)) {
   AUDIO_DEBUG ("Instance and stream NULL in Npp_writeReady");
   DEBUG_PRINT_FUNC_EXIT
   return NPERR_INVALID_INSTANCE_ERROR;
 }

 AudioPlugin* pPlugin = static_cast<AudioPlugin*>(instance->pdata);

 if (pPlugin) {
   AUDIO_DEBUG ("pPlugin->writeReady !!! Called");
    iMaxNum = pPlugin->writeReady(stream);
  }
  DEBUG_PRINT_FUNC_EXIT
  return iMaxNum;
}

int32_t NPP_Audio_Write(NPP instance,
                NPStream *stream,
                int32_t offset,
                int32_t len,
                void *buffer) {
  DEBUG_PRINT_FUNC_ENTRY

  int32_t iRetLen = -1;

  if ((instance == NULL) ||
      (stream == NULL) ||
      (offset < 0) ||
      (len <= 0)||(buffer == NULL))  {
    AUDIO_DEBUG(" NPP Write Fail ");
    DEBUG_PRINT_FUNC_EXIT
    return NPERR_INVALID_INSTANCE_ERROR;
  }

  AudioPlugin * pPlugin = static_cast<AudioPlugin*>(instance->pdata);

  if (pPlugin) {
    iRetLen = pPlugin->write(stream, offset, len, buffer);
  }

  DEBUG_PRINT_FUNC_EXIT
  return iRetLen;
}

NPError NPP_Audio_DestroyStream(NPP instance, NPStream *stream, NPReason reason) {
  DEBUG_PRINT_FUNC_ENTRY
  AUDIO_DEBUG("Reason code for destroy : %d ", reason);

  NPError rv = NPERR_GENERIC_ERROR;

  if (instance == NULL) {
    AUDIO_DEBUG("Invalid instance in destroy stream ");
    DEBUG_PRINT_FUNC_EXIT
    return  NPERR_INVALID_INSTANCE_ERROR;
  }

  if (stream == NULL)  {
    AUDIO_DEBUG("Stream NUll in destroy stream");
    DEBUG_PRINT_FUNC_EXIT
    return  NPERR_GENERIC_ERROR;
  }

  AudioPlugin * pPlugin = static_cast<AudioPlugin*>(instance->pdata);

  if (!pPlugin) {
    rv = NPERR_INVALID_PLUGIN_ERROR;
  }  else {
    rv = pPlugin->destroyStream(stream, reason);
  }
  DEBUG_PRINT_FUNC_EXIT
  return rv;
}

void NPP_Audio_StreamAsFile(NPP instance, NPStream* stream, const char* fname) {
  DEBUG_PRINT_FUNC_ENTRY
  DEBUG_PRINT_FUNC_EXIT
/* nothing to do */
}

void NPP_Audio_Print(NPP instance, NPPrint* printInfo) {
  DEBUG_PRINT_FUNC_ENTRY
  DEBUG_PRINT_FUNC_EXIT
/* nothing to do */
}

void NPP_Audio_URLNotify(NPP instance,
                   const char* url,
                   NPReason reason, 
                   void* notifyData)
{
  DEBUG_PRINT_FUNC_ENTRY

  if (instance == NULL) {
    DEBUG_PRINT_FUNC_EXIT
    return;
  }

  if (url == NULL) {
    DEBUG_PRINT_FUNC_EXIT
    return;
  }
  DEBUG_PRINT_FUNC_EXIT
}

#if NP_VERSION_MAJOR==0 && NP_VERSION_MINOR<=11
#define NPP void*
#endif

NPError NPP_Audio_SetValue(NPP instance, NPNVariable variable, void *value) {
  DEBUG_PRINT_FUNC_ENTRY

  if (instance == NULL) {
    DEBUG_PRINT_FUNC_EXIT
    return NPERR_INVALID_INSTANCE_ERROR;
  }

  DEBUG_PRINT_FUNC_EXIT
  return NPERR_NO_ERROR;
}

#if NP_VERSION_MAJOR==0 && NP_VERSION_MINOR<=11
#undef NPP
#endif

int16_t NPP_Audio_HandleEvent(NPP instance, void* event) {
  DEBUG_PRINT_FUNC_ENTRY
  if (instance == NULL)
  {
    DEBUG_PRINT_FUNC_EXIT
    return 0;
  }
  DEBUG_PRINT_FUNC_EXIT
  return 0;
}

NPObject *NPP_Audio__GetScriptableInstance(NPP instance) {
  DEBUG_PRINT_FUNC_ENTRY
  if (instance == NULL) {
    DEBUG_PRINT_FUNC_EXIT
    return 0;
  }

  NPObject *npobj = 0;
  DEBUG_PRINT_FUNC_ENTRY
  return npobj;
}

#ifdef __cplusplus
}
#endif
