/*****************************************************************************
 * Copyright 2010 Sony Corporation. All rights reserved.
 *****************************************************************************/

/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is 
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or 
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

////////////////////////////////////////////////////////////
//
// Implementation of Netscape entry points (NPN_*)
//

#include <glib.h>
#include <glib/gprintf.h>

#include "npapi.h"
#include "npfunctions.h"
#include "webbr_log.h"

#undef DEBUG
#undef DEBUG_PRINT
#undef DEBUG_PRINT_FUNC_ENTRY
#undef DEBUG_PRINT_FUNC_EXIT

#if DEBUG
#define DEBUG_PRINT_FUNC_ENTRY {\
     WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_INFO,"BROADCAST_PLUGIN::%s[%d] Entry", __FUNCTION__, __LINE__); }
#define DEBUG_PRINT_FUNC_EXIT {\
     WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_INFO,"BROADCAST_PLUGIN::%s[%d] Exit", __FUNCTION__, __LINE__); }

#else
#define DEBUG_PRINT_FUNC_ENTRY
#define DEBUG_PRINT_FUNC_EXIT
#endif

#ifndef HIBYTE
#define HIBYTE(x) ((((uint32_t)(x)) & 0xff00) >> 8)
#endif

#ifndef LOBYTE
#define LOBYTE(W) ((W) & 0xFF)
#endif

extern NPNetscapeFuncs NPNFuncs;

void NPN_Version(int* plugin_major,
                 int* plugin_minor,
                 int* netscape_major,
                 int* netscape_minor) {
  DEBUG_PRINT_FUNC_ENTRY
  *plugin_major   = NP_VERSION_MAJOR;
  *plugin_minor   = NP_VERSION_MINOR;
  *netscape_major = HIBYTE(NPNFuncs.version);
  *netscape_minor = LOBYTE(NPNFuncs.version);
}

NPError NPN_GetURLNotify(NPP instance,
                         const gchar *url,
                         const gchar *target,
                         void* notifyData) {
  DEBUG_PRINT_FUNC_ENTRY
  gint navMinorVers = NPNFuncs.version & 0xFF;
  NPError rv = NPERR_NO_ERROR;

  if ( navMinorVers >= NPVERS_HAS_NOTIFICATION )
    rv = NPNFuncs.geturlnotify(instance, url, target, notifyData);
  else
    rv = NPERR_INCOMPATIBLE_VERSION_ERROR;

  return rv;
}

NPError NPN_PostURL(NPP instance,
                    const gchar* url,
                    const gchar* window,
                    uint32_t len,
                    const gchar* buf,
                    NPBool file) {
  DEBUG_PRINT_FUNC_ENTRY
  NPError rv = NPNFuncs.posturl(instance, url, window, len, buf, file);
  return rv;
}

NPError NPN_GetURL(NPP instance, const gchar *url, const gchar *target) {
  DEBUG_PRINT_FUNC_ENTRY
  NPError rv = NPNFuncs.geturl(instance, url, target);
  return rv;
}

NPError NPN_PostURLNotify(NPP instance,
                          const gchar* url,
                          const gchar* window,
                          uint32_t len,
                          const gchar* buf,
                          NPBool file,
                          void* notifyData)  {
  DEBUG_PRINT_FUNC_ENTRY
  gint navMinorVers = NPNFuncs.version & 0xFF;
  NPError rv = NPERR_NO_ERROR;
  if ( navMinorVers >= NPVERS_HAS_NOTIFICATION )
    rv = NPNFuncs.posturlnotify(instance, url, window, len, buf, file, notifyData);
  else
    rv = NPERR_INCOMPATIBLE_VERSION_ERROR;

  return rv;
}

NPError NPN_RequestRead(NPStream* stream, NPByteRange* rangeList) {
  DEBUG_PRINT_FUNC_ENTRY
  NPError rv = NPNFuncs.requestread(stream, rangeList);
  return rv;
}

NPError NPN_NewStream(NPP instance,
                      NPMIMEType type,
                      const gchar* target,
                      NPStream** stream) {
  DEBUG_PRINT_FUNC_ENTRY
  gint navMinorVersion = NPNFuncs.version & 0xFF;

  NPError rv = NPERR_NO_ERROR;

  if ( navMinorVersion >= NPVERS_HAS_STREAMOUTPUT )
    rv = NPNFuncs.newstream(instance, type, target, stream);
  else
    rv = NPERR_INCOMPATIBLE_VERSION_ERROR;

  return rv;
}

int32_t NPN_Write(NPP instance, NPStream *stream, int32_t len, void *buffer) {
  DEBUG_PRINT_FUNC_ENTRY
  gint navMinorVersion = NPNFuncs.version & 0xFF;
  int32_t rv = 0;

  if ( navMinorVersion >= NPVERS_HAS_STREAMOUTPUT )
    rv = NPNFuncs.write(instance, stream, len, buffer);
  else
    rv = -1;
  return rv;
}

NPError NPN_DestroyStream(NPP instance, NPStream* stream, NPError reason) {
  DEBUG_PRINT_FUNC_ENTRY
  gint navMinorVersion = NPNFuncs.version & 0xFF;
  NPError rv = NPERR_NO_ERROR;

  if ( navMinorVersion >= NPVERS_HAS_STREAMOUTPUT)
    rv = NPNFuncs.destroystream(instance, stream, reason);
  else
    rv = NPERR_INCOMPATIBLE_VERSION_ERROR;
  return rv;
}

void NPN_Status(NPP instance, const gchar *message) {
  DEBUG_PRINT_FUNC_ENTRY
  NPNFuncs.status(instance, message);
}

const gchar* NPN_UserAgent(NPP instance) {
  DEBUG_PRINT_FUNC_ENTRY
  const gchar * rv = NULL;
  rv = NPNFuncs.uagent(instance);
  return rv;
}

void* NPN_MemAlloc(uint32_t size) {
  DEBUG_PRINT_FUNC_ENTRY
  void * rv = NULL;
  rv = NPNFuncs.memalloc(size);
  return rv;
}

void NPN_MemFree(void* ptr) {
  DEBUG_PRINT_FUNC_ENTRY
  NPNFuncs.memfree(ptr);
}

uint32_t NPN_MemFlush(uint32_t size) {
  DEBUG_PRINT_FUNC_ENTRY
  uint32_t rv = NPNFuncs.memflush(size);
  return rv;
}

void NPN_ReloadPlugins(NPBool reloadPages) {
  DEBUG_PRINT_FUNC_ENTRY
  NPNFuncs.reloadplugins(reloadPages);
}

NPError NPN_GetValue(NPP instance, NPNVariable variable, void *value) {
  DEBUG_PRINT_FUNC_ENTRY
  NPError rv = NPNFuncs.getvalue(instance, variable, value);
  return rv;
}

NPError NPN_SetValue(NPP instance, NPPVariable variable, void *value) {
  DEBUG_PRINT_FUNC_ENTRY
#if 0
  NPError rv = NPNFuncs.setvalue(instance, variable, value);
  return rv;
#else
  NPNFuncs.setvalue(instance, variable, value);
  return NPERR_NO_ERROR;
#endif
}

void NPN_InvalidateRect(NPP instance, NPRect *invalidRect) {
  DEBUG_PRINT_FUNC_ENTRY
  NPNFuncs.invalidaterect(instance, invalidRect);
}

void NPN_InvalidateRegion(NPP instance, NPRegion invalidRegion) {
  DEBUG_PRINT_FUNC_ENTRY
  NPNFuncs.invalidateregion(instance, invalidRegion);
}

void NPN_ForceRedraw(NPP instance) {
  DEBUG_PRINT_FUNC_ENTRY
  NPNFuncs.forceredraw(instance);
}

NPIdentifier NPN_GetStringIdentifier(const NPUTF8 *name) {
  DEBUG_PRINT_FUNC_ENTRY
  return NPNFuncs.getstringidentifier(name);
}

void NPN_GetStringIdentifiers(const NPUTF8 **names,
                              uint32_t nameCount,
                              NPIdentifier *identifiers)  {
  DEBUG_PRINT_FUNC_ENTRY
  return NPNFuncs.getstringidentifiers(names, nameCount, identifiers);
}

NPIdentifier NPN_GetIntIdentifier(int32_t intid) {
  DEBUG_PRINT_FUNC_ENTRY
  return NPNFuncs.getintidentifier(intid);
}

bool NPN_IdentifierIsString(NPIdentifier identifier) {
  DEBUG_PRINT_FUNC_ENTRY
  return NPNFuncs.identifierisstring(identifier);
}

NPUTF8 *NPN_UTF8FromIdentifier(NPIdentifier identifier) {
  DEBUG_PRINT_FUNC_ENTRY
  return NPNFuncs.utf8fromidentifier(identifier);
}

int32_t NPN_IntFromIdentifier(NPIdentifier identifier) {
  DEBUG_PRINT_FUNC_ENTRY
  return NPNFuncs.intfromidentifier(identifier);
}

NPObject *NPN_CreateObject(NPP npp, NPClass *aClass) {
  DEBUG_PRINT_FUNC_ENTRY
  return NPNFuncs.createobject(npp, aClass);
}

NPObject *NPN_RetainObject(NPObject *obj) {
  DEBUG_PRINT_FUNC_ENTRY
  return NPNFuncs.retainobject(obj);
}

void NPN_ReleaseObject(NPObject *obj) {
  DEBUG_PRINT_FUNC_ENTRY
  NPNFuncs.releaseobject(obj);
}

bool NPN_Invoke(NPP npp, NPObject* obj,
                NPIdentifier methodName,
                const NPVariant *args,
                uint32_t argCount,
                NPVariant *result) {
  DEBUG_PRINT_FUNC_ENTRY
  return NPNFuncs.invoke(npp, obj, methodName, args, argCount, result);
}

bool NPN_InvokeDefault(NPP npp,
                       NPObject* obj,
                       const NPVariant *args,
                       uint32_t argCount,
                       NPVariant *result) {
  DEBUG_PRINT_FUNC_ENTRY
  return NPNFuncs.invokeDefault(npp, obj, args, argCount, result);
}

bool NPN_Evaluate(NPP npp,
                  NPObject* obj,
                  NPString *script,
                  NPVariant *result) {
  DEBUG_PRINT_FUNC_ENTRY
  return NPNFuncs.evaluate(npp, obj, script, result);
}

bool NPN_GetProperty(NPP npp, NPObject* obj,
                     NPIdentifier propertyName,
                     NPVariant *result) {
  DEBUG_PRINT_FUNC_ENTRY
  return NPNFuncs.getproperty(npp, obj, propertyName, result);
}

bool NPN_SetProperty(NPP npp,
                     NPObject* obj,
                     NPIdentifier propertyName,
                     const NPVariant *value) {
  DEBUG_PRINT_FUNC_ENTRY
  return NPNFuncs.setproperty(npp, obj, propertyName, value);
}

bool NPN_RemoveProperty(NPP npp, NPObject* obj, NPIdentifier propertyName) {
  DEBUG_PRINT_FUNC_ENTRY
  return NPNFuncs.removeproperty(npp, obj, propertyName);
}

bool NPN_HasProperty(NPP npp, NPObject* obj, NPIdentifier propertyName) {
  DEBUG_PRINT_FUNC_ENTRY
  return NPNFuncs.hasproperty(npp, obj, propertyName);
}

bool NPN_HasMethod(NPP npp, NPObject* obj, NPIdentifier methodName) {
  DEBUG_PRINT_FUNC_ENTRY
  return NPNFuncs.hasmethod(npp, obj, methodName);
}

void NPN_ReleaseVariantValue(NPVariant *variant) {
  DEBUG_PRINT_FUNC_ENTRY
  NPNFuncs.releasevariantvalue(variant);
}

void NPN_SetException(NPObject* obj, const NPUTF8 *message) {
  DEBUG_PRINT_FUNC_ENTRY
  NPNFuncs.setexception(obj, message);
}
