/*****************************************************************************
 * Copyright 2010 Sony Corporation. All rights reserved.
 *****************************************************************************/

/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is 
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or 
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

////////////////////////////////////////////////////////////
//
// Implementation of plugin entry points (NPP_*)
// most are just empty stubs for this particular plugin
//

#include <glib.h>
#include <glib/gprintf.h>
#include "npapi.h"
#include "npfunctions.h"
#include "npruntime.h"
#include "broadcast_plugin.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _ValuePointerObject {
        NPObject object;
        NPP instance;
} ValuePointerObject;

static NPIdentifier vp_channel;

static NPObject *vp_allocate(NPP npp, NPClass *aClass) {
  DEBUG_PRINT_FUNC_ENTRY
  ValuePointerObject* obj;

  vp_channel = NPN_GetStringIdentifier("channel");

  obj = (ValuePointerObject *)NPN_MemAlloc(sizeof(ValuePointerObject));
  if ( obj == NULL )
    return NULL;
  obj->instance = npp;

  DEBUG_PRINT_FUNC_EXIT
  return (NPObject*) obj;
}

static void vp_deallocate(NPObject *npobj) {
  DEBUG_PRINT_FUNC_ENTRY
  ValuePointerObject* obj = (ValuePointerObject*) npobj;
  if (obj) {
    NPN_MemFree(obj);
  }
  DEBUG_PRINT_FUNC_EXIT
}

static bool vp_has_property(NPObject *npobj, NPIdentifier name) {
  DEBUG_PRINT_FUNC_ENTRY
  bool result = (name == vp_channel);
  DEBUG_PRINT_FUNC_EXIT
  return result;
}

static bool vp_get_property(NPObject *npobj, NPIdentifier name, NPVariant *result) {
  DEBUG_PRINT_FUNC_ENTRY
  // MEMO: dummy
  INT32_TO_NPVARIANT(10, *result);
  DEBUG_PRINT_FUNC_EXIT
  return true;
}

static bool vp_set_property(NPObject *npobj, NPIdentifier name, const NPVariant *value) {
  DEBUG_PRINT_FUNC_ENTRY

  DEBUG_PRINT_FUNC_EXIT
  return true;
}

/** for Scripting Plugin  **/
struct NPClass ValuePointerClass = {
  NP_CLASS_STRUCT_VERSION,
  vp_allocate,
  vp_deallocate,
  NULL,
  NULL,
  NULL,
  NULL,
  vp_has_property,
  vp_get_property,
  vp_set_property,
  NULL
};



/** DTP Plugin Manager */
//DtpPluginManager* g_pDtpPluginMgr = NULL;

NPObject*  GetValue(NPP instance, NPPVariable variable);

gchar* NPP_GetMIMEDescription(void)  {
  DEBUG_PRINT_FUNC_ENTRY
  return (gchar*)BROADCAST_PLUGIN_MIME;  // BROADCAST MIME
}

NPError NPP_New(NPMIMEType pluginType,
                NPP instance,
                uint16_t mode,
                int16_t argc,
                gchar* argn[],
                gchar* argv[],
                NPSavedData* saved) {
  DEBUG_PRINT_FUNC_ENTRY
  NPError err = NPERR_NO_ERROR;
  DEBUG_PRINT(g_strdup_printf("Type=%s,instance=%p,mode=%d\n", pluginType, instance, mode));

  if (instance == NULL) {
    DEBUG_PRINT_FUNC_EXIT
    return NPERR_INVALID_INSTANCE_ERROR;
  }

  NPObject* winObj;
  err = NPN_GetValue(instance, NPNVWindowNPObject, &winObj);
  if (err != NPERR_NO_ERROR) {
    DEBUG_PRINT_FUNC_EXIT
    return err;
  }

  NPVariant res;
  NPString script;
  const char *buf = "opera.pip_support;";
  script.UTF8Characters = buf;
  script.UTF8Length = strlen(buf);
  if (NPN_Evaluate(instance, winObj, &script, &res)) {
    if (res.type == NPVariantType_Bool) {
      if (res.value.boolValue) {
        DEBUG_PRINT(g_strdup_printf("opera.pip_support is TRUE"));
      } else {
        DEBUG_PRINT(g_strdup_printf("opera.pip_support is FALSE"));
        DEBUG_PRINT_FUNC_EXIT
        return NPERR_INVALID_PARAM;
      }
    } else {
      DEBUG_PRINT(g_strdup_printf("Result is not NPVariantType_Bool"));
      DEBUG_PRINT_FUNC_EXIT
      return NPERR_INVALID_PARAM;
    }
  } else {
    DEBUG_PRINT(g_strdup_printf("Fail NPN_Evaluate"));
    DEBUG_PRINT_FUNC_EXIT
    return NPERR_INVALID_PARAM;
  }

  BroadcastPlugin *pPlugin = new BroadcastPlugin(instance);
  if (pPlugin == NULL) {
    DEBUG_PRINT_FUNC_EXIT;
    return NPERR_OUT_OF_MEMORY_ERROR;
  }
  instance->pdata = (void *)pPlugin;

  /* windowless mode setting */
  err = NPN_SetValue(instance, NPPVpluginWindowBool, (void*)false);

  DEBUG_PRINT_FUNC_EXIT
  return err;
}

NPError NPP_Destroy(NPP instance, NPSavedData** save) {
  DEBUG_PRINT_FUNC_ENTRY

  if (instance == NULL) {
    DEBUG_PRINT_FUNC_EXIT
    return NPERR_INVALID_INSTANCE_ERROR;
  }
  NPError rv = NPERR_NO_ERROR;
  BroadcastPlugin *pPlugin = static_cast<BroadcastPlugin*>(instance->pdata);
  if (pPlugin) {
    rv = pPlugin->StopBroadcast();
  }

  DEBUG_PRINT_FUNC_EXIT
  return rv;
}

NPError NPP_SetWindow(NPP instance, NPWindow* pNPWindow)  {
  DEBUG_PRINT_FUNC_ENTRY

  if (instance == NULL) {
    DEBUG_PRINT_FUNC_EXIT
    return NPERR_INVALID_INSTANCE_ERROR;
  }
  if (pNPWindow == NULL) {
    DEBUG_PRINT_FUNC_EXIT
    return NPERR_GENERIC_ERROR;
  }
  NPError rv = NPERR_NO_ERROR;
  BroadcastPlugin *pPlugin = static_cast<BroadcastPlugin*>(instance->pdata);
  if (pPlugin) {
    rv = pPlugin->SetBroadcastRegion(pNPWindow);
  }

  DEBUG_PRINT_FUNC_EXIT
  return rv;
}

#if NP_VERSION_MAJOR == 0 && NP_VERSION_MINOR <= 11
#define NPP void*
#endif


NPError NPP_GetValue(NPP instance, NPPVariable variable, void *value) {
  NPError rv = NPERR_GENERIC_ERROR;
  DEBUG_PRINT_FUNC_ENTRY

  DEBUG_PRINT(g_strdup_printf("Instance=%p", instance));

  switch (variable) {
    case NPPVpluginNameString:
      *((gchar **)value) = BROADCAST_PLUGIN_NAME;
      DEBUG_PRINT(("PluginName"));
    break;
    case NPPVpluginDescriptionString:
      *((gchar **)value) = BROADCAST_PLUGIN_DESCRIPTION;
      DEBUG_PRINT((" Plugin Description "));
    break;
    case NPPVpluginScriptableNPObject:
      DEBUG_PRINT(g_strdup_printf("NpObject_var:%d  NpObjectVal: %p ", variable, *(NPObject **)value));
      NPObject* npObject;
      npObject = NPN_CreateObject(instance, &ValuePointerClass);
      if ( npObject == NULL )
        return NPERR_OUT_OF_MEMORY_ERROR;
      *((NPObject**)value) = npObject;
      return NPERR_NO_ERROR;
    break;
    default:
      rv = NPERR_GENERIC_ERROR;
      DEBUG_PRINT_FUNC_EXIT
      return NPERR_GENERIC_ERROR;
    break;
  }

  DEBUG_PRINT_FUNC_EXIT
  return rv;
}

#if NP_VERSION_MAJOR == 0 && NP_VERSION_MINOR <= 11
#undef NPP
#endif

//
// NPP_NewStream()
//
NPError NPP_NewStream(NPP instance,
        NPMIMEType type,
        NPStream* stream,
        NPBool seekable,
        uint16_t* stype) {
  DEBUG_PRINT_FUNC_ENTRY
  if (instance == NULL) {
    return NPERR_INVALID_INSTANCE_ERROR;
  }
  if (type == NULL) {
    return NPERR_INVALID_INSTANCE_ERROR;
  }
  if (stream == NULL) {
    return NPERR_INVALID_INSTANCE_ERROR;
  }
  NPError rv = NPERR_GENERIC_ERROR;

  DEBUG_PRINT_FUNC_EXIT
  return rv;
}

int32_t NPP_WriteReady(NPP instance, NPStream *stream)  {
  DEBUG_PRINT_FUNC_ENTRY
  const int32_t error = (int32_t)-1;
  if (instance == NULL) {
    return error;
  }
  if (stream == NULL) {
    return error;
  }
  int32_t bufferSize = error;

  DEBUG_PRINT(g_strdup_printf("broadcast plugin buffsize:%d\n", (int)bufferSize));
  return bufferSize;
}


int32_t NPP_Write(NPP instance,
                NPStream *stream,
                int32_t offset,
                int32_t len,
                void *buffer)  {
  DEBUG_PRINT_FUNC_ENTRY

  const int32_t error = (int32_t)-1;

  if (instance == NULL) {
    return error;
  }
  if (stream == NULL) {
    return error;
  }
  if (offset < 0) {
    return error;
  }
  if (len <= 0) {
    return error;
  }
  if (buffer == NULL) {
    return error;
  }
  int32_t rv = error;

  DEBUG_PRINT_FUNC_EXIT
  return rv;
}


NPError NPP_DestroyStream(NPP instance, NPStream *stream, NPReason reason)  {
  DEBUG_PRINT_FUNC_ENTRY
  if (instance == NULL) {
    return NPERR_INVALID_INSTANCE_ERROR;
  }
  if (stream == NULL) {
    return NPERR_GENERIC_ERROR;
  }
  NPError rv = NPERR_GENERIC_ERROR;

  DEBUG_PRINT_FUNC_EXIT
  return rv;
}

void NPP_StreamAsFile(NPP instance, NPStream* stream, const gchar* fname)  {
/* nothing to do */
}

void NPP_Print(NPP instance, NPPrint* printInfo)  {
/* nothing to do */
}

void NPP_URLNotify(NPP instance, const gchar* url, NPReason reason, void* notifyData)  {
  DEBUG_PRINT_FUNC_ENTRY
  if (instance == NULL) {
    return;
  }
  if (url == NULL) {
    return;
  }

  DEBUG_PRINT_FUNC_EXIT
}

#if NP_VERSION_MAJOR == 0 && NP_VERSION_MINOR <= 11
#define NPP void*
#endif

NPError NPP_SetValue(NPP instance, NPNVariable variable, void *value) {
  DEBUG_PRINT_FUNC_ENTRY
  if (instance == NULL) {
    return NPERR_INVALID_INSTANCE_ERROR;
  }
  NPError rv = NPERR_NO_ERROR;

  DEBUG_PRINT_FUNC_EXIT
  return rv;
}

#if NP_VERSION_MAJOR == 0 && NP_VERSION_MINOR <= 11
#undef NPP
#endif


int16_t NPP_HandleEvent(NPP instance, void* event)  {
  DEBUG_PRINT_FUNC_ENTRY
  if (instance == NULL) {
    return 0;
  }

  NPError rv = NPERR_NO_ERROR;
  BroadcastPlugin *pPlugin = static_cast<BroadcastPlugin*>(instance->pdata);
  if (pPlugin) {
    rv = pPlugin->StartBroadcast();
  }

  DEBUG_PRINT_FUNC_EXIT
  return rv;
}


NPObject *NPP_GetScriptableInstance(NPP instance) {
  DEBUG_PRINT_FUNC_ENTRY

  if (instance == NULL) {
    return 0;
  }

  NPObject *npobj = 0;

  DEBUG_PRINT_FUNC_EXIT
  return npobj;
}

#ifdef __cplusplus
}
#endif
