/*****************************************************************************
 * Copyright 2010 Sony Corporation. All rights reserved.
 *****************************************************************************/

/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is 
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or 
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

////////////////////////////////////////////////////////////
//
// Implementation of plugin entry points (NPP_*)
// most are just empty stubs for this particular plugin
//

#include <glib.h>
#include <glib/gprintf.h>
#include "dtp_plugin.h"
#include "npapi.h"
#include "npfunctions.h"
#include "webbr_log.h"

#ifdef __cplusplus
extern "C" {
#endif

#undef DEBUG
#undef DEBUG_PRINT_FUNC_ENTRY
#undef DEBUG_PRINT_FUNC_EXIT

#define DEBUG 1

#if DEBUG
#define DEBUG_PRINT_FUNC_ENTRY {\
     WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_INFO,"NPP::%s Entry Line:%d", __FUNCTION__, __LINE__); }
#define DEBUG_PRINT_FUNC_EXIT {\
     WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_INFO,"NPP::%s Exit Line:%d", __FUNCTION__, __LINE__); }

#else
#define DEBUG_PRINT_FUNC_ENTRY
#define DEBUG_PRINT_FUNC_EXIT
#endif

/** DTP Plugin State */
static const gint OBJECT_NUM_MAX	= 8 * (5 + 1);
static int32_t g_ObjectNum = 0;

NPObject*  GetValue(NPP instance, NPPVariable variable);

gchar* NPP_GetMIMEDescription(void)  {
  DEBUG_PRINT_FUNC_ENTRY
  WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_INFO,"DTP_PLUGIN_MIME=%s\n", DTP_PLUGIN_MIME_TYPE_DESCRIPTION);

  return (gchar*)DTP_PLUGIN_MIME_TYPE_DESCRIPTION;  // DTP Mime
}

NPError NPP_Initialize(void)  {
//  NPError err = NPERR_GENERIC_ERROR;
  DEBUG_PRINT_FUNC_ENTRY
  g_ObjectNum = 0;
  DEBUG_PRINT_FUNC_EXIT
  return NPERR_NO_ERROR;
}

void NPP_Shutdown(void) {
  DEBUG_PRINT_FUNC_ENTRY
  g_ObjectNum = 0;
  DEBUG_PRINT_FUNC_EXIT
}

NPError NPP_New(NPMIMEType pluginType,
                NPP instance,
                uint16_t mode,
                int16_t argc,
                gchar* argn[],
                gchar* argv[],
                NPSavedData* saved) {
  DEBUG_PRINT_FUNC_ENTRY
  WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_DETAIL1,"NPP::%s Line:%d- Type=%s,instance=%p,mode=%d\n", __FUNCTION__, __LINE__, pluginType, instance, mode);

  if (instance == NULL) {
    DEBUG_PRINT_FUNC_EXIT
    return NPERR_INVALID_INSTANCE_ERROR;
  }

// Check OBJECT_NUM_MAX
  if (g_ObjectNum >= OBJECT_NUM_MAX) {
    WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_INFO,"ERROR: Object too many!!\n");
    return NPERR_GENERIC_ERROR;
  }
  g_ObjectNum++;
  WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_INFO,"g_ObjectNum = %d\n",g_ObjectNum);

  DtpPlugin * pPlugin = new DtpPlugin(instance);
  if (pPlugin == NULL) {
    g_ObjectNum--;
    DEBUG_PRINT_FUNC_EXIT
    WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_WARNING,"NPP::%s Line:%d- NPERR_OUT_OF_MEMORY_ERROR \n", __FUNCTION__, __LINE__);
    return NPERR_OUT_OF_MEMORY_ERROR;
  }

  instance->pdata = (void *)pPlugin; //use pPlugin->init()!

  NPError rv = pPlugin->init();
  if (rv != NPERR_NO_ERROR) {
    delete pPlugin;
    g_ObjectNum--;
    instance->pdata = NULL;
    DEBUG_PRINT_FUNC_EXIT
    return rv;
  }

  pPlugin->getScriptableInstance();

  WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_INFO,"NPP_New Exit: ObjectNum[%d] instance[%p] pPlugin[%p]\n",g_ObjectNum,instance, pPlugin);
  //DEBUG_PRINT_FUNC_EXIT
  return NPERR_NO_ERROR;
}

NPError NPP_Destroy(NPP instance, NPSavedData** save) {
  DEBUG_PRINT_FUNC_ENTRY
  NPError err = NPERR_NO_ERROR;

  if (instance == NULL) {
    DEBUG_PRINT_FUNC_EXIT
    return NPERR_INVALID_INSTANCE_ERROR;
  }

  DtpPlugin * pPlugin = static_cast<DtpPlugin*>(instance->pdata);

  if (pPlugin) {
    err = pPlugin->shutdown();
    if (g_ObjectNum <= 0) {
      WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_DETAIL1,"DecreaseObjectNum=%d failed.\n",g_ObjectNum);
      g_ObjectNum = 0;
    }
    else {
      g_ObjectNum--;
      WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_DETAIL1,"g_ObjectNum = %d delete in\n", g_ObjectNum);
    }
    instance->pdata = NULL;
    delete pPlugin;
  }
  WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_DETAIL1,"NPP_Destroy Exit: ObjectNum[%d] instance[%p] pPlugin[%p]\n",g_ObjectNum, instance, pPlugin);
  //DEBUG_PRINT_FUNC_EXIT
  return NPERR_NO_ERROR;
}

NPError NPP_SetWindow(NPP instance, NPWindow* pNPWindow)  {
  DEBUG_PRINT_FUNC_ENTRY
  if (instance == NULL) {
    DEBUG_PRINT_FUNC_EXIT
    return NPERR_INVALID_INSTANCE_ERROR;
  }
  if (pNPWindow == NULL) {
    DEBUG_PRINT_FUNC_EXIT
    return NPERR_GENERIC_ERROR;
  }
  NPError rv = NPERR_NO_ERROR;
  DtpPlugin * pPlugin = static_cast<DtpPlugin*>(instance->pdata);
  if (pPlugin) {
    rv = pPlugin->setWindow(pNPWindow);
  }
  DEBUG_PRINT_FUNC_EXIT
  return rv;
}

#if NP_VERSION_MAJOR == 0 && NP_VERSION_MINOR <= 11
#define NPP void*
#endif


NPError NPP_GetValue(NPP instance, NPPVariable variable, void *value) {
  NPError rv = NPERR_NO_ERROR;
  DEBUG_PRINT_FUNC_ENTRY

  if (instance == NULL) {
  WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_WARNING,"\n NPP::%s Line:%d- NPP_GetValue instance fail \n", __FUNCTION__, __LINE__);
  return NPERR_INVALID_INSTANCE_ERROR;
  }

  WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_DETAIL1,"NPP::%s Line:%d- Instance=%p\n", __FUNCTION__, __LINE__, instance);

  switch (variable) {
    case NPPVpluginNameString:
      *((gchar **)value) = DTP_PLUGIN_NAME;
      WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_INFO,"NPP::%s Line:%d- PluginName \n", __FUNCTION__, __LINE__);
    break;
    case NPPVpluginDescriptionString:
      *((gchar **)value) = DTP_PLUGIN_DESCRIPTION;
      WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_INFO,"NPP::%s Line:%d-  Plugin Description \n", __FUNCTION__, __LINE__);
    break;
    case NPPVpluginScriptableNPObject:
      *(NPObject **)value =  GetValue(instance, variable);
      WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_DETAIL1,"\n NPP::%s Line:%d- NpObject_var:%d  NpObjectVal: %p  \n", __FUNCTION__, __LINE__, variable, *(NPObject **)value);
    break;
    default:
      rv = NPERR_GENERIC_ERROR;
      DEBUG_PRINT_FUNC_EXIT
      return NPERR_GENERIC_ERROR;
    break;
  }

  DEBUG_PRINT_FUNC_EXIT
  return rv;
}

NPObject*  GetValue(NPP instance, NPPVariable variable) {
  NPObject* object_value = NULL;
  DEBUG_PRINT_FUNC_ENTRY

  DtpPlugin * pPlugin = static_cast<DtpPlugin*>(instance->pdata);
  if (pPlugin) {
    object_value = pPlugin->getScriptableInstance();
  } else {
    WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_DETAIL1,"\n NPP::%s Line:%d-  base getvalue fail \n", __FUNCTION__, __LINE__);
  }

  DEBUG_PRINT_FUNC_EXIT
  return object_value;
}


#if NP_VERSION_MAJOR == 0 && NP_VERSION_MINOR <= 11
#undef NPP
#endif

//
// NPP_NewStream()
//
NPError NPP_NewStream(NPP instance,
        NPMIMEType type,
        NPStream* stream,
        NPBool seekable,
        uint16_t* stype) {
  DEBUG_PRINT_FUNC_ENTRY
  if (instance == NULL) {
    return NPERR_INVALID_INSTANCE_ERROR;
  }
  if (type == NULL) {
    return NPERR_INVALID_INSTANCE_ERROR;
  }
  if (stream == NULL) {
    return NPERR_INVALID_INSTANCE_ERROR;
  }
  NPError rv = NPERR_GENERIC_ERROR;
  DtpPlugin * pPlugin = static_cast<DtpPlugin*>(instance->pdata);
  if (pPlugin) {
    rv = pPlugin->newStream(type, stream, seekable, stype);
  }
  DEBUG_PRINT_FUNC_EXIT
  return rv;
}

int32_t NPP_WriteReady(NPP instance, NPStream *stream)  {
  DEBUG_PRINT_FUNC_ENTRY
  const int32_t error = (int32_t)-1;
  if (instance == NULL) {
    return error;
  }
  if (stream == NULL) {
    return error;
  }
  int32_t bufferSize = error;
  DtpPlugin * pPlugin = static_cast<DtpPlugin*>(instance->pdata);
  if (pPlugin) {
    bufferSize = pPlugin->writeReady(stream);
  }
  //  DEBUG_PRINT_FUNC_EXIT
  WbrPrintf(DT_WEBBR_NPAPI,DT_LEVEL_DETAIL1,"NPP::%s Line:%d-  dtp plugin buffsize:%d\n", __FUNCTION__, __LINE__, (int)bufferSize);
  return bufferSize;
}


int32_t NPP_Write(NPP instance,
                NPStream *stream,
                int32_t offset,
                int32_t len,
                void *buffer)  {
  DEBUG_PRINT_FUNC_ENTRY

  const int32_t error = (int32_t)-1;

  if (instance == NULL) {
    return error;
  }
  if (stream == NULL) {
    return error;
  }
  if (offset < 0) {
    return error;
  }
  if (len <= 0) {
    return error;
  }
  if (buffer == NULL) {
    return error;
  }
  int32_t rv = error;

  DtpPlugin * pPlugin = static_cast<DtpPlugin*>(instance->pdata);
  if (pPlugin) {
  rv = pPlugin->write(stream, offset, len, buffer);
  }
  DEBUG_PRINT_FUNC_EXIT
  return rv;
}


NPError NPP_DestroyStream(NPP instance, NPStream *stream, NPReason reason)  {
  DEBUG_PRINT_FUNC_ENTRY
  if (instance == NULL) {
    return NPERR_INVALID_INSTANCE_ERROR;
  }
  if (stream == NULL) {
    return NPERR_GENERIC_ERROR;
  }
  NPError rv = NPERR_GENERIC_ERROR;

  DtpPlugin * pPlugin = static_cast<DtpPlugin*>(instance->pdata);
  if (pPlugin) {
  rv = pPlugin->destroyStream(stream, reason);
  }
  DEBUG_PRINT_FUNC_EXIT
  return rv;
}

void NPP_StreamAsFile(NPP instance, NPStream* stream, const gchar* fname)  {
/* nothing to do */
}

void NPP_Print(NPP instance, NPPrint* printInfo)  {
/* nothing to do */
}

void NPP_URLNotify(NPP instance, const gchar* url, NPReason reason, void* notifyData)  {
  DEBUG_PRINT_FUNC_ENTRY
  if (instance == NULL) {
    return;
  }
  if (url == NULL) {
    return;
}

  DtpPlugin * pPlugin = static_cast<DtpPlugin*>(instance->pdata);
  if (pPlugin) {
  pPlugin->urlNotify(url, reason, notifyData);
  }
  DEBUG_PRINT_FUNC_EXIT
}

#if NP_VERSION_MAJOR == 0 && NP_VERSION_MINOR <= 11
#define NPP void*
#endif

NPError NPP_SetValue(NPP instance, NPNVariable variable, void *value) {
  DEBUG_PRINT_FUNC_ENTRY
  if (instance == NULL) {
    return NPERR_INVALID_INSTANCE_ERROR;
  }
  NPError rv = NPERR_NO_ERROR;

  DtpPlugin * pPlugin = static_cast<DtpPlugin*>(instance->pdata);
  if (pPlugin) {
  rv = pPlugin->setValue(variable, value);
  }
  DEBUG_PRINT_FUNC_EXIT
  return rv;
}

#if NP_VERSION_MAJOR == 0 && NP_VERSION_MINOR <= 11
#undef NPP
#endif


int16_t NPP_HandleEvent(NPP instance, void* event)  {
  if (instance == NULL) {
    return 0;
  }
  int16_t rv = 0;

  DtpPlugin * pPlugin = static_cast<DtpPlugin*>(instance->pdata);
  if (pPlugin) {
  rv = pPlugin->handleEvent(event);
  }
  
  return rv;
}


NPObject *NPP_GetScriptableInstance(NPP instance) {
  DEBUG_PRINT_FUNC_ENTRY

  if (instance == NULL) {
    return 0;
  }

  NPObject *npobj = 0;

  DtpPlugin * pPlugin = static_cast<DtpPlugin*>(instance->pdata);
  if (pPlugin) {
  npobj = pPlugin->getScriptableInstance();
  }
  DEBUG_PRINT_FUNC_EXIT
  return npobj;
}

#ifdef __cplusplus
}
#endif
