/*****************************************************************************
 * Copyright 2012 Sony Corporation. All rights reserved.
 *****************************************************************************/

/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is 
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or 
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

////////////////////////////////////////////////////////////
//
// Implementation of plugin entry points (NPP_*)
// most are just empty stubs for this particular plugin
//

#include <glib.h>
#include <glib/gprintf.h>
#include <string.h>
#include "npapi.h"
#include "npfunctions.h"
#include "npruntime.h"
#include "vumu_plugin.h"

#include "gogi_plugin_api.h"
#include "opera_mgr.h"

typedef struct _ValuePointerObject {
  NPObject object;
  NPP instance;
  NPObject *callback;
  VUMUPlugin *plugin;
} ValuePointerObject;

/*
 * callback to javascript
 */
void doCallback(NPP npp, NPObject *callback, int returnCode, int msgID) {
  DEBUG_PRINT_FUNC_ENTRY;
  NPVariant args[3];
  uint32_t argCount = 3;
  NPVariant *result = NULL;

  char* resultMsg;
  int resultCode;
  switch(returnCode) {
  case 0:
    resultMsg = g_strdup("Successful\0");
    resultCode = 0;
    break;
  case 1:
    resultMsg = g_strdup("Unknown error\0");
    resultCode = 1;
    break;
  case 2:
    resultMsg = g_strdup("Cannot process request\0");
    resultCode = 2;
    break;
  case RETURN_CODE_MIME_UNKNOWN_ERROR:
    resultMsg = g_strdup("Unknown MIME type\0");
    resultCode = 3;
    break;
  case 4:
    resultMsg = g_strdup("User Consent Needed\0");
    resultCode = 4;
    break;
  default:
    resultMsg = g_strdup("Unknown error\0");
    resultCode = 1;
    break;
  }

  INT32_TO_NPVARIANT(msgID, args[0]);
  STRINGZ_TO_NPVARIANT(resultMsg, args[1]);
  INT32_TO_NPVARIANT(resultCode, args[2]);
  NPN_InvokeDefault(npp, callback, args, argCount, result);
}

/*
 * ipc register callback for bsc_ovb
 */
static void setCallbackIPC(ValuePointerObject* obj) {
  DEBUG_PRINT_FUNC_ENTRY;

  obj->plugin = new VUMUPlugin(obj->instance);

  DEBUG_PRINT("*npp: %p, *callback: %p, plugin: %p", &obj->instance, obj->callback, obj->plugin);
  obj->plugin->emitRegisterCallbackDrm();

  DEBUG_PRINT_FUNC_EXIT;
}

static int sendDRMMessage(ValuePointerObject* obj, char* msgType, char* msg, char* DRMSystemID) {
  DEBUG_PRINT_FUNC_ENTRY;
  DEBUG_PRINT("type: %s, system: %s", msgType, DRMSystemID);
  DEBUG_PRINT("msg: %s", msg);

  static int msgID = 0;
  msgID++;
  DEBUG_PRINT("msgID: %d", msgID);
  obj->plugin->setMsgID(msgID);

  if (strncmp(DEFAULT_MARLIN_MSG_TYPE, msgType, strlen(DEFAULT_MARLIN_MSG_TYPE))) {
    obj->plugin->callCallback(RETURN_CODE_MIME_UNKNOWN_ERROR);
  } else {
    DEBUG_PRINT("EMIT OVB_GET_DRM_LICENSE: requestObj %p ", obj->plugin);
    obj->plugin->emitGetDRMLicense(msg);
  }

  DEBUG_PRINT_FUNC_EXIT;
  return msgID;
}

static void deletePlugin(ValuePointerObject *obj) {
  if ((obj != NULL) && (obj->plugin != NULL)) {
    obj->plugin->deletePlugin();
  }
}

#ifdef __cplusplus
extern "C" {
#endif

  static NPIdentifier vp_onDRMMessageResult;
  static NPIdentifier vp_sendDrmMessage;

  static NPObject *vp_allocate(NPP npp, NPClass *aClass) {
    DEBUG_PRINT_FUNC_ENTRY;
    ValuePointerObject* obj;

    vp_onDRMMessageResult = NPN_GetStringIdentifier("onDRMMessageResult");
    vp_sendDrmMessage = NPN_GetStringIdentifier("sendDRMMessage");

    obj = (ValuePointerObject *)NPN_MemAlloc(sizeof(ValuePointerObject));
    if ( obj == NULL )
      return NULL;
    obj->instance = npp;

    setCallbackIPC(obj);

    DEBUG_PRINT_FUNC_EXIT;
    return (NPObject*) obj;
  }

  static void vp_deallocate(NPObject *npobj) {
    DEBUG_PRINT_FUNC_ENTRY;

    ValuePointerObject* obj = (ValuePointerObject*) npobj;
    deletePlugin(obj);
    if (obj != NULL) {
      NPN_MemFree(obj);
    }
    DEBUG_PRINT_FUNC_EXIT;
  }

  static bool vp_has_method(NPObject *npobj, NPIdentifier name) {
    DEBUG_PRINT_FUNC_ENTRY;
    bool result = false;
    if (!result)
      result = (name == vp_sendDrmMessage);
    DEBUG_PRINT_FUNC_EXIT;
    return result;
  }

  static bool vp_invoke(NPObject *npobj, NPIdentifier name,
                         const NPVariant *args, uint32_t argCount, NPVariant *result) {
    DEBUG_PRINT_FUNC_ENTRY;
    ValuePointerObject* obj = (ValuePointerObject*) npobj;

    if ((name == vp_sendDrmMessage) && (argCount == 3)) {
      int msgID = sendDRMMessage(obj,
                                 g_strndup(NPVARIANT_TO_STRING(args[0]).UTF8Characters,
                                           NPVARIANT_TO_STRING(args[0]).UTF8Length),
                                 g_strndup(NPVARIANT_TO_STRING(args[1]).UTF8Characters,
                                           NPVARIANT_TO_STRING(args[1]).UTF8Length),
                                 g_strndup(NPVARIANT_TO_STRING(args[2]).UTF8Characters,
                                           NPVARIANT_TO_STRING(args[2]).UTF8Length));

      INT32_TO_NPVARIANT(msgID, *result);
    }
    DEBUG_PRINT_FUNC_EXIT;
    return true;
  }

  static bool vp_has_property(NPObject *npobj, NPIdentifier name) {
    DEBUG_PRINT_FUNC_ENTRY;
    bool result = (name == vp_onDRMMessageResult);
    DEBUG_PRINT_FUNC_EXIT;
    return result;
  }

  static bool vp_get_property(NPObject *npobj, NPIdentifier name, NPVariant *result) {
    DEBUG_PRINT_FUNC_ENTRY;
    ValuePointerObject* obj = (ValuePointerObject*) npobj;

    if (name == vp_onDRMMessageResult) {
      DEBUG_PRINT("get callback function: onDRMMessageResult.");
      OBJECT_TO_NPVARIANT(obj->callback, *result);
    }

    DEBUG_PRINT_FUNC_EXIT;
    return true;
  }

  static bool vp_set_property(NPObject *npobj, NPIdentifier name, const NPVariant *value) {
    DEBUG_PRINT_FUNC_ENTRY;
    ValuePointerObject* obj = (ValuePointerObject*) npobj;

    if (name == vp_onDRMMessageResult) {
      DEBUG_PRINT("set callback function: onDRMMessageResult.");
      obj->callback = NPVARIANT_TO_OBJECT(*value);
      obj->plugin->setCallback(obj->callback);
    }

    DEBUG_PRINT_FUNC_EXIT;
    return true;
  }

  /** for Scripting Plugin  **/
  struct NPClass ValuePointerClass = {
    NP_CLASS_STRUCT_VERSION,
    vp_allocate,
    vp_deallocate,
    NULL,
    vp_has_method,
    vp_invoke,
    NULL,
    vp_has_property,
    vp_get_property,
    vp_set_property,
    NULL
  };

  NPObject*  GetValue(NPP instance, NPPVariable variable);

  gchar* NPP_GetMIMEDescription(void)  {
    DEBUG_PRINT_FUNC_ENTRY;
    return VUMU_PLUGIN_MIME;  // VUMU MIME
  }

  NPError NPP_New(NPMIMEType pluginType,
                  NPP instance,
                  uint16_t mode,
                  int16_t argc,
                  gchar* argn[],
                  gchar* argv[],
                  NPSavedData* saved) {
    DEBUG_PRINT_FUNC_ENTRY;
    NPError err = NPERR_NO_ERROR;
    DEBUG_PRINT("Type=%s,instance=%p,mode=%d\n", pluginType, instance, mode);

    if (instance == NULL) {
      DEBUG_PRINT_FUNC_EXIT;
      return NPERR_INVALID_INSTANCE_ERROR;
    }

    /* windowless mode setting */
    err = NPN_SetValue(instance, NPPVpluginWindowBool, (void*)false);

    DEBUG_PRINT_FUNC_EXIT;
    return err;
  }

  NPError NPP_Destroy(NPP instance, NPSavedData** save) {
    DEBUG_PRINT_FUNC_ENTRY;

    if (instance == NULL) {
      DEBUG_PRINT_FUNC_EXIT;
      return NPERR_INVALID_INSTANCE_ERROR;
    }

    DEBUG_PRINT_FUNC_EXIT;
    return NPERR_NO_ERROR;
  }

  NPError NPP_SetWindow(NPP instance, NPWindow* pNPWindow)  {
    DEBUG_PRINT_FUNC_ENTRY;
    if (instance == NULL) {
      DEBUG_PRINT_FUNC_EXIT;
      return NPERR_INVALID_INSTANCE_ERROR;
    }
    if (pNPWindow == NULL) {
      DEBUG_PRINT_FUNC_EXIT;
      return NPERR_GENERIC_ERROR;
    }
    NPError rv = NPERR_NO_ERROR;

    DEBUG_PRINT_FUNC_EXIT;
    return rv;
  }

#if NP_VERSION_MAJOR == 0 && NP_VERSION_MINOR <= 11
#define NPP void*
#endif


  NPError NPP_GetValue(NPP instance, NPPVariable variable, void *value) {
    NPError rv = NPERR_GENERIC_ERROR;
    DEBUG_PRINT_FUNC_ENTRY;

    DEBUG_PRINT("Instance=%p", instance);

    switch (variable) {
    case NPPVpluginNameString:
      *((gchar **)value) = VUMU_PLUGIN_NAME;
      DEBUG_PRINT("PluginName");
      break;
    case NPPVpluginDescriptionString:
      *((gchar **)value) = VUMU_PLUGIN_DESCRIPTION;
      DEBUG_PRINT(" Plugin Description ");
      break;
    case NPPVpluginScriptableNPObject:
      DEBUG_PRINT("NpObject_var:%d  NpObjectVal: %p ", variable, *(NPObject **)value);
      NPObject* npObject;
      npObject = NPN_CreateObject(instance, &ValuePointerClass);
      if ( npObject == NULL )
        return NPERR_OUT_OF_MEMORY_ERROR;
      *((NPObject**)value) = npObject;
      return NPERR_NO_ERROR;
      break;
    default:
      rv = NPERR_GENERIC_ERROR;
      DEBUG_PRINT_FUNC_EXIT;
      return NPERR_GENERIC_ERROR;
      break;
    }

    DEBUG_PRINT_FUNC_EXIT;
    return rv;
  }

#if NP_VERSION_MAJOR == 0 && NP_VERSION_MINOR <= 11
#undef NPP
#endif

  //
  // NPP_NewStream()
  //
  NPError NPP_NewStream(NPP instance,
                        NPMIMEType type,
                        NPStream* stream,
                        NPBool seekable,
                        uint16_t* stype) {
    DEBUG_PRINT_FUNC_ENTRY;
    if (instance == NULL) {
      return NPERR_INVALID_INSTANCE_ERROR;
    }
    if (type == NULL) {
      return NPERR_INVALID_INSTANCE_ERROR;
    }
    if (stream == NULL) {
      return NPERR_INVALID_INSTANCE_ERROR;
    }
    NPError rv = NPERR_GENERIC_ERROR;

    DEBUG_PRINT_FUNC_EXIT;
    return rv;
  }

  int32_t NPP_WriteReady(NPP instance, NPStream *stream)  {
    DEBUG_PRINT_FUNC_ENTRY;
    const int32_t error = (int32_t)-1;
    if (instance == NULL) {
      return error;
    }
    if (stream == NULL) {
      return error;
    }
    int32_t bufferSize = error;

    DEBUG_PRINT("vumu plugin buffsize:%d\n", (int)bufferSize);
    return bufferSize;
  }


  int32_t NPP_Write(NPP instance,
                    NPStream *stream,
                    int32_t offset,
                    int32_t len,
                    void *buffer)  {
    DEBUG_PRINT_FUNC_ENTRY;

    const int32_t error = (int32_t)-1;

    if (instance == NULL) {
      return error;
    }
    if (stream == NULL) {
      return error;
    }
    if (offset < 0) {
      return error;
    }
    if (len <= 0) {
      return error;
    }
    if (buffer == NULL) {
      return error;
    }
    int32_t rv = error;

    DEBUG_PRINT_FUNC_EXIT;
    return rv;
  }


  NPError NPP_DestroyStream(NPP instance, NPStream *stream, NPReason reason)  {
    DEBUG_PRINT_FUNC_ENTRY;
    if (instance == NULL) {
      return NPERR_INVALID_INSTANCE_ERROR;
    }
    if (stream == NULL) {
      return NPERR_GENERIC_ERROR;
    }
    NPError rv = NPERR_GENERIC_ERROR;

    DEBUG_PRINT_FUNC_EXIT;
    return rv;
  }

  void NPP_StreamAsFile(NPP instance, NPStream* stream, const gchar* fname)  {
    /* nothing to do */
  }

  void NPP_Print(NPP instance, NPPrint* printInfo)  {
    /* nothing to do */
  }

  void NPP_URLNotify(NPP instance, const gchar* url, NPReason reason, void* notifyData)  {
    DEBUG_PRINT_FUNC_ENTRY;
    if (instance == NULL) {
      return;
    }
    if (url == NULL) {
      return;
    }

    DEBUG_PRINT_FUNC_EXIT;
  }

#if NP_VERSION_MAJOR == 0 && NP_VERSION_MINOR <= 11
#define NPP void*
#endif

  NPError NPP_SetValue(NPP instance, NPNVariable variable, void *value) {
    DEBUG_PRINT_FUNC_ENTRY;
    if (instance == NULL) {
      return NPERR_INVALID_INSTANCE_ERROR;
    }
    NPError rv = NPERR_NO_ERROR;

    DEBUG_PRINT_FUNC_EXIT;
    return rv;
  }

#if NP_VERSION_MAJOR == 0 && NP_VERSION_MINOR <= 11
#undef NPP
#endif


  int16_t NPP_HandleEvent(NPP instance, void* event)  {
    /* 2012.9.11: disable logging for preventing huge log flow */
    /* DEBUG_PRINT_FUNC_ENTRY; */
    if (instance == NULL) {
      return 0;
    }
    int16_t rv = 0;

    /*
      GogiPluginEvent *pEvent = static_cast<GogiPluginEvent*>(event);
      DEBUG_PRINT(g_strdup_printf("gogi event: %d", pEvent->type));
    */

    /* 2012.9.11: disable logging for preventing huge log flow */
    /* DEBUG_PRINT_FUNC_EXIT; */
    return rv;
  }


  NPObject *NPP_GetScriptableInstance(NPP instance) {
    DEBUG_PRINT_FUNC_ENTRY;

    if (instance == NULL) {
      return 0;
    }

    NPObject *npobj = 0;

    DEBUG_PRINT_FUNC_EXIT;
    return npobj;
  }

#ifdef __cplusplus
}
#endif
